//
// tournament - creation of playing schedules for tournaments
// Copyright (C) 2006,2009,2010  Joachim Reichel <joachim.reichel@gmx.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

/// \file
/// \brief The #Timer class.

#ifndef TIMER_H
#define TIMER_H

#ifndef _WIN32
#define HAVE_SYS_TIME_H
#define HAVE_SYS_RESOURCE_H
#endif // _WIN32

#if defined (HAVE_SYS_TIME_H)
#  include <sys/time.h>
#  if defined (HAVE_SYS_RESOURCE_H)
#    include <sys/resource.h>
#  endif // HAVE_SYS_RESOURCE_H
#else
#  include <ctime>
#endif // HAVE_SYS_TIME_H



/// Simple timer.

/// \class Timer
/// The timer can be started and stopped using #start() and #stop(). Calls of
/// start() are ignored if the timer is already running, likewise calls of
/// stop() if the timer is not running. At any time get_real_time() can be
/// called to get the real time (in seconds) accumulated up to now.
/// Analogously, #get_user_time() and #get_system_time() return the user time
/// and the system time, respectively. The function #reset() may be called at
/// any time to stop and reset the timer. The function #restart() is equivalent
/// to calling #reset() and #start().
/// \warning Make sure not to call timer related functions too often. It has
///          been observed that calling #get_user_time() about thousand
///          times a second might extremely slow down your program,
///          depending on the architecture.
/// \warning The class only works properly if the header files \c <sys/time.h>
///          and \c <sys/resource.h> are present. If one or both files are
///          missing, #get_user_time() measures \e real time and
///          #get_system_time() always returns zero.
/// \warning If both header files are missing, \c time() is used to measure the
///          time.
class Timer {

  public:

    /// Constructor
    Timer();

    /// Starts the timer
    /// This function starts the timer
    /// (ignored if the timer is already running).
    void start();

    /// Stops the timer
    /// This function stops the timer
    /// (ignored if the timer is not running).
    void stop();

    /// Resets the timer
    /// This function stops the timer and resets it to zero.
    void reset();

    /// Resets the timer
    /// This function resets the timer to zero and starts it.
    void restart();

    /// Returns the accumulated real time up to now
    double get_real_time();

    /// Returns the accumulated user time up to now
    double get_user_time();

    /// Returns the accumulated system time up to now
    double get_system_time();


 private:

#if defined (HAVE_SYS_TIME_H)

    /// Computes the time interval (in seconds) between the points of
    /// time \a t1 and \a t2
    static double interval (const timeval& t1, const timeval& t2);

#endif // HAVE_SYS_TIME_H

    /// Accumulated real time up to the last call of #stop() or
    /// #get_real_time()
    double  total_real_time;

    /// Accumulated user time up to the last call of #stop() or
    /// #get_user_time()
    double  total_user_time;

    /// Accumulated system time up to the last call of #stop() or
    /// # get_system_time()
    double  total_system_time;

#if defined (HAVE_SYS_TIME_H)

    /// Time stamp (real time)for the last call of #start() or
    /// #get_real_time()
    timeval start_real_time;

    /// Time stamp (user time) for the last call of #start() or
    /// #get_user_time()
    timeval start_user_time;

    /// Time stamp (system time) for the last call of #start() or
    /// #get_system_time()
    timeval start_system_time;

#else

    /// Time stamp for the last call of #start() (real time)
    double start_real_time;

    /// Time stamp for the last call of #start() (user time)
    double start_user_time;

    /// Time stamp for the last call of #start() (system time)
    double start_system_time;

#endif // HAVE_SYS_TIME_H

    /// Indicates whether the timer is running at the moment
    bool    running;

};



#endif // TIMER_H
